import requests
import os

from .utils.logger import getLogger

LOGGER = getLogger(__name__)

def download_file(url, filepath):
    LOGGER.debug(f"Downloading file from {url} to path: {filepath}")
    os.makedirs(os.path.dirname(filepath), exist_ok=True)
    
    response = requests.get(url, timeout=60, verify=False)
    
    if response.status_code == 200:
        with open(filepath, "wb") as file:
            file.write(response.content)
        LOGGER.debug(f"File downloaded successfully at {filepath}")
    else:
        LOGGER.error(f"Error downloading file: {response.status_code}")
    
    


import logging 
import os
import sys
from ..config import CONFIG

def getLogger(name):
    os.makedirs(CONFIG.LOGS_PATH, exist_ok=True)
    logging.basicConfig(level=logging.DEBUG, 
                        handlers=[logging.FileHandler(CONFIG.LOGS_FILE),
                                  logging.StreamHandler(sys.stdout)],
                        format=CONFIG.LOGS_FORMAT)
    logger = logging.getLogger(name)
    return logger
    


import os

from .config import CONFIG
from .filedownloader import download_file

from .utils.logger import getLogger

LOGGER = getLogger(__name__)

class FishbowlURLManager:
    __headers = None

    def get_cert_path(self):
        LOGGER.debug(CONFIG.CA_CERT_FILEPATH)
        if os.path.exists(CONFIG.CA_CERT_FILEPATH):
            return CONFIG.CA_CERT_FILEPATH
        download_file(CONFIG.CA_CERT_LOCATION, CONFIG.CA_CERT_FILEPATH)
        LOGGER.debug(CONFIG.CA_CERT_FILEPATH)
        return CONFIG.CA_CERT_FILEPATH
    
    def get_headers(self, session_key):
        if not self.__headers:
            self.__headers = {'Accept': '*/*',
                   'Connection': 'keep-alive',
                   'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64)',
                   'session-key': session_key
                   }
        return self.__headers
    
    def get_bowl_details_url(self, bowl_name):
        return f"https://api.fishbowlapp.com/v4/feed/{bowl_name}?v=2&addNetworkingImages=true"
    
    def get_posts_url(self, bowl_id, sort:str='byDate', start:int=0, count:int=20):
        return f"https://api.fishbowlapp.com/v4/feed/{bowl_id}/posts?sort={sort}&skipSystemMessages=true&start={start}&count={count}"


import requests
from .utils.logger import getLogger
from .urlmanager import FishbowlURLManager

LOGGER = getLogger(__name__)

class FishBowlAPI:
    __session_key = None
    __url_manager = None
    def __init__(self, session_key: str):
        LOGGER.debug("Creating fishbowlapi object")
        self.__session_key = session_key
        self.__url_manager = FishbowlURLManager()

    def get_posts(self, bowl_name:str=None, sort:str=None, start:int=None, count:int=None):
        """_summary_

        Args:
            bowl_name (str): _description_

        1. Get the bowl details using the bowl name
        2. Fetch the bowl id and call posts api to get the posts
        """
        kwargs = {k:v for k,v in locals().items() if v is not None and k not in ['self', 'bowl_name']}
        bowl_details = self.get_bowl_details(bowl_name)
        bowl_id = bowl_details['_id']
        LOGGER.debug(bowl_id)
        posts = self.get_posts_by_bowl_id(bowl_id, **kwargs)
        return posts


    def get_bowl_details(self, bowl_name):
        _url = self.__url_manager.get_bowl_details_url(bowl_name)
        _headers = self.__url_manager.get_headers(self.__session_key)
        _cert = self.__url_manager.get_cert_path()
        data = requests.get(url=_url, headers=_headers, verify=_cert, timeout=60)
        return dict(data.json())

    def get_posts_by_bowl_id(self, bowl_id, sort:str=None, start:int=None, count:int=None):
        kwargs = {k:v for k,v in locals().items() if v is not None and k not in ['self', 'bowl_id']}
        _url = self.__url_manager.get_posts_url(bowl_id, **kwargs)
        _headers = self.__url_manager.get_headers(self.__session_key)
        _cert = self.__url_manager.get_cert_path()
        LOGGER.debug(_url)
        data = requests.get(url=_url, headers=_headers, verify=_cert, timeout=60)
        LOGGER.debug(len(dict(data.json())))
        return dict(data.json())

    def get_comments(self):
        pass


from selenium import webdriver
from selenium.webdriver.chromium.webdriver import ChromiumDriver
from webdriver_manager.chrome import ChromeDriverManager
from webdriver_manager.microsoft import EdgeChromiumDriverManager

from .drivertype import DriverType

from .utils.logger import getLogger

LOGGER = getLogger(__name__)


class BrowserDriver:
    __driver = None
    __service = None
    __driver_type = None
    __driver_path = None
    
    def __init__(self, driver_type=DriverType.CHROME_DRIVER, driver_path:str=None) -> None:
        if driver_path:
            self.__driver_path = driver_path
        if driver_type:
            self.__driver_type = driver_type
        if not driver_path:
            self.__driver_path = self.get_default_driver(self.__driver_type)
        if driver_type == DriverType.CHROME_DRIVER:
            self.__service = webdriver.ChromeService(
                executable_path=self.__driver_path)
            self.__driver = webdriver.Chrome(service=self.__service)
        elif driver_type == DriverType.EDGE_DRIVER:
            self.__service = webdriver.EdgeService(
                executable_path=self.__driver_path)
            self.__driver = webdriver.Edge(service=self.__service)

    def get_driver(self) -> ChromiumDriver:
        return self.__driver

    def get_default_driver(self, driver_type) -> ChromiumDriver:
        try:
            if driver_type == DriverType.CHROME_DRIVER:
                return ChromeDriverManager().install()
            elif driver_type == DriverType.EDGE_DRIVER:
                return EdgeChromiumDriverManager().install()
        except Exception as e:
            LOGGER.error(f"Error getting default driver {e}")


import sys
sys.path.append('../src')

from fishbowlpy.fishbowlloginmanager import FishBowlLoginManager

client = FishBowlLoginManager()
#client.login(driver_type=DriverType.EDGE_DRIVER)
print(client)



import sys
sys.path.append('../src')

from fishbowlpy.fishbowlapi import FishBowlAPI

api = FishBowlAPI("c0a616e2-078c-4563-bddc-36f19d032513")
#api.get_bowl_details('tech-india')
api.get_posts('tech-india')


import json
import os
import time
from fishbowlpy.browserdriver import BrowserDriver
from fishbowlpy.config import CONFIG
from fishbowlpy.drivertype import DriverType
from .utils.logger import getLogger

LOGGER = getLogger(__name__)

class FishBowlLoginManager:
    __session_key = None
    __session_expiry = None
    __driver = None

    def __init__(
        self,
        session_key: str = None,
        session_expiry: float = None,
        login_popup: bool = True,
        driver_type: str = DriverType.CHROME_DRIVER,
        driver_path: str = None
    ):
        """Initialize a FishBowlLoginManager object with a session key and session expiry.
        If session_key is not provided, tries to retreive the session from previously
        logged in session. If login_popup is True, attempts to login manually.

        Args:
            session_key (str, optional): session key from cookie. Defaults to None.
            session_expiry (float, optional): session expiry in epoch seconds. Defaults to None.
            login_popup (bool, optional): Indicates whether login popup should be opened. Defaults to True.
            driver_type (str, optional): Driver type if login popup is True. Defaults to DriverType.CHROME_DRIVER
        """
        LOGGER.debug("Creating the login manager...")
        if session_key:
            self.__session_key = session_key
        if session_expiry:
            self.__session_expiry = session_expiry
        LOGGER.debug("Attempting to load session...")
        logged_in = self.load_session()
        if not logged_in and login_popup:
            if driver_type:
                self.login(driver_type, driver_path=driver_path)
            else:
                self.login()

    def login(self, driver_type: str = DriverType.CHROME_DRIVER, driver_path: str = None):
        """Fishbowl client logs in by either reading previous session or by manually
        logging in.

        Args:
            driver_type (str, optional): Provide the driver type. Defaults to DriverType.CHROME_DRIVER.
        """
        if self.load_session():
            return

        if not self.__driver:
            self.__driver = BrowserDriver(driver_type=driver_type, 
                                          driver_path=driver_path).get_driver()
        self.__driver.get(url=CONFIG.FISHBOWLAPP_LOGIN_URL)

        while True:
            LOGGER.debug("Attempting to fetch cookie...")
            cookie = self.__driver.get_cookie(CONFIG.SESSION_KEY_COOKIE_NAME)

            if cookie and cookie.get(CONFIG.SESSION_KEY_COOKIE_DOMAIN) == CONFIG.SESSION_KEY_COOKIE_DOMAIN_FISHBOWL:
                LOGGER.debug(f"Fetching cookie...{cookie}")
                self.__session_key = cookie.get(CONFIG.SESSION_KEY_COOKIE_VALUE)
                self.__session_expiry = cookie.get(CONFIG.SESSION_KEY_COOKIE_EXPIRY)
                self.save_session_data()
                break
            time.sleep(CONFIG.LOGIN_SLEEP_DURATION)
        LOGGER.debug(f"Logged in successfully - {self.__session_key}")
        self.__driver.quit()

    def __str__(self) -> str:
        """Representation of the FishBowlLoginManager instance.

        Returns:
            str: String representation of the FishBowlLoginManager
        """      
        return f"""FishBowlLoginManager(session_key: {self.__session_key}, session_expiry: {self.__session_expiry})"""

    def save_session_data(self):
        """Saves the session data into a json file for future login attempts"""
        data = {
            CONFIG.SESSION_KEY_COOKIE_VALUE: self.__session_key,
            CONFIG.SESSION_KEY_COOKIE_EXPIRY: self.__session_expiry,
        }
        os.makedirs(os.path.dirname(CONFIG.SESSION_FILE), exist_ok=True)
        with open(CONFIG.SESSION_FILE, "w", encoding="utf-8") as session_file:
            json.dump(data, session_file)

    def load_session(self, file=CONFIG.SESSION_FILE) -> bool:
        LOGGER.debug(f"Loading session from file: {file}")
        session_data = None
        try:
            with open(file, "r", encoding="utf-8") as session_file:
                session_data = json.load(session_file)
        except FileNotFoundError as e:
            LOGGER.error(f"File not found {e}")
        if not session_data:
            return False
        
        if session_data[CONFIG.SESSION_KEY_COOKIE_EXPIRY] < time.time():
            LOGGER.error("Session has Expired")
            return False
        self.__session_key = session_data[CONFIG.SESSION_KEY_COOKIE_VALUE]
        self.__session_expiry = session_data[CONFIG.SESSION_KEY_COOKIE_EXPIRY]

        return True

    def set_session_key(self, session_key=None, session_expiry=None):
        """Sets the session key and session expiry

        Args:
            session_key (str, optional): The session key from cookie. Defaults to None.
            session_expiry (int, optional): The epoch time when session expires. Defaults to None.
        """        
        if session_expiry:
            self.__session_expiry = session_expiry
        if session_key:
            self.__session_key = session_key
    
    def get_session_key(self) -> str:
        """Returns the session key

        Returns:
            str: Session key for current session
        """        
        return self.__session_key


import os


class CONFIG:
    CHROME_DRIVER_PATH = "D:\\My Open Source Contributions\\FishbowlApp\\drivers\\chromedriver.exe"
    EDGE_DRIVER_PATH = "D:\\My Open Source Contributions\\FishbowlApp\\drivers\\msedgedriver.exe"

    FISHBOWLAPP_URL = "https://fishbowlapp.com"
    FISHBOWLAPP_LOGIN_URL = "https://fishbowlapp.com/login"

    LOGIN_SLEEP_DURATION = 5

    SESSION_KEY_COOKIE_NAME = 'session_key'
    SESSION_KEY_COOKIE_VALUE = 'value'
    SESSION_KEY_COOKIE_EXPIRY = 'expiry'
    SESSION_KEY_COOKIE_DOMAIN = 'domain'
    SESSION_KEY_COOKIE_DOMAIN_FISHBOWL = '.fishbowlapp.com'

    SESSION_FILE = os.path.join(os.path.dirname(__file__), "tmp", "session", "session_data.json")

    GET_BOWLS_URL = FISHBOWLAPP_URL + "/bowls"
    GET_POSTS_URL = "https://api.fishbowlapp.com/v4/feed/60f19b15fc523b002f774065/posts?sort=byDate&skipSystemMessages=true&start=0&count=20"
    
    CA_CERT_LOCATION = "https://raw.githubusercontent.com/mukulbindal/fishbowlpy/main/resources/certs/Zscaler%20Root%20CA.crt"
    CA_CERT_FILEPATH = os.path.join(os.path.dirname(__file__), 'tmp', 'certs','Zscaler Root CA.crt')

    LOGS_PATH = os.path.join(os.path.dirname(__file__),'tmp','logs')
    LOGS_FILE = os.path.join(LOGS_PATH, 'fishbowlpy.log')
    LOGS_FORMAT = "[%(asctime)s] [%(levelname)s] [%(process)d] [%(module)s.%(funcName)s] [%(lineno)d]: %(message)s"



import sys
sys.path.append("../src")

from fishbowlpy.fishbowlclient import FishBowlClient
from fishbowlpy.drivertype import DriverType

client = FishBowlClient(driver_type=DriverType.EDGE_DRIVER)

posts = client.get_posts('tech-india')

print(posts)


from .fishbowlloginmanager import FishBowlLoginManager
from .fishbowlapi import FishBowlAPI
from .drivertype import DriverType
from .utils.logger import getLogger

LOGGER = getLogger(__name__)


class FishBowlClient:
    __login_manager:FishBowlLoginManager = None
    __fishbowl_api: FishBowlAPI = None

    def __init__(self, login_manager:FishBowlLoginManager=None,
                 session_key: str = None,
                 session_expiry: float = None,
                 login_popup: bool = True,
                 driver_type: str = DriverType.CHROME_DRIVER,
                 fishbowl_api: FishBowlAPI = None
                 ) -> None:
        
        if login_manager:
            self.__login_manager = login_manager
        else:
            self.__login_manager = FishBowlLoginManager(session_expiry=session_expiry, 
                                                        session_key=session_key, 
                                                        login_popup=login_popup, 
                                                        driver_type=driver_type)
        if fishbowl_api:
            self.__fishbowl_api = fishbowl_api
        else:
            self.__fishbowl_api = FishBowlAPI(session_key=self.__login_manager.get_session_key())
    
    def refresh_session(self):
        pass
    def get_bowls_names(self):
        # Hard coded for now
        # It shoould use the bs4 to parse html and get the proper names
        return ['tech-india', 'job-referrals']
    
    def get_posts(self, bowl_name):
        return self.__fishbowl_api.get_posts(bowl_name=bowl_name)
    
        